package com.yeejoin.amos.boot.module.jg.biz.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yeejoin.amos.boot.biz.common.bo.ReginParams;
import com.yeejoin.amos.boot.biz.common.controller.BaseController;
import com.yeejoin.amos.boot.module.jg.api.dto.CodeGenerateDto;
import com.yeejoin.amos.boot.module.jg.api.dto.UseFlagParamDto;
import com.yeejoin.amos.boot.module.jg.api.entity.JgRegistrationHistory;
import com.yeejoin.amos.boot.module.jg.api.mapper.JgUseRegistrationMapper;
import com.yeejoin.amos.boot.module.jg.api.service.IJgInstallationNoticeService;
import com.yeejoin.amos.boot.module.jg.biz.service.ICommonService;
import com.yeejoin.amos.boot.module.jg.biz.service.IPdfService;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.JgRegistrationHistoryServiceImpl;
import com.yeejoin.amos.boot.module.jg.biz.utils.CodeUtil;
import com.yeejoin.amos.boot.module.ymt.api.entity.EquipmentCategory;
import com.yeejoin.amos.boot.module.ymt.api.enums.FlowStatusEnum;
import com.yeejoin.amos.feign.systemctl.Systemctl;
import com.yeejoin.amos.feign.systemctl.model.DictionarieValueModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 监管公共Api
 *
 * @author system_generator
 * @date 2021-10-20
 */
@RestController
@Api(tags = "监管公共Api")
@RequestMapping(value = "/common")
public class CommonController extends BaseController {
    @Autowired
    ICommonService commonService;

    @Autowired
    IPdfService pdfService;

    @Autowired
    JgRegistrationHistoryServiceImpl jgRegistrationHistoryService;

    @Autowired
    JgUseRegistrationMapper jgUseRegistrationMapper;

    @Autowired
    CodeUtil codeUtil;

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getChildren")
    @ApiOperation(httpMethod = "GET", value = "通过设备种类code获取设备类别", notes = "通过设备种类code获取设备类别")
    public ResponseModel<List<EquipmentCategory>> getChildren(@RequestParam(value = "code") String code,
                                                              @RequestParam(value = "type", required = false) String type) {
        return ResponseHelper.buildResponse(commonService.getEquipmentCategoryList(code, type));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getRegion")
    @ApiOperation(httpMethod = "GET", value = "获取行政区划", notes = "获取行政区划")
    public ResponseModel<Object> getRegion(@RequestParam(value = "level", required = false) String level,
                                           @RequestParam(value = "parentId", required = false) String parentId) {
        return ResponseHelper.buildResponse(commonService.getRegion(level, parentId));
    }

    /**
     * 获取管辖分局树
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/creatTree")
    @ApiOperation(httpMethod = "GET", value = "获取管辖分局树", notes = "获取管辖分局树")
    public ResponseModel<Object> creatTree() {
        return ResponseHelper.buildResponse(commonService.getTree(getSelectedOrgInfo()));
    }


    /**
     * 工作台接收机构树
     *
     * @param bizType 业务类型：notice-告知类， register-登记类
     * @return 树
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/{bizType}/receive-unit/tree")
    @ApiOperation(httpMethod = "GET", value = "接收机构树", notes = "接收机构树")
    public ResponseModel<Object> getNoticeReceiveTree(@PathVariable String bizType) {
        return ResponseHelper.buildResponse(commonService.getReceiveUnitTree(bizType));
    }


    /**
     * 获取行政审批局树
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getApproveTree")
    @ApiOperation(httpMethod = "GET", value = "获取行政审批局树", notes = "获取行政审批局树")
    public ResponseModel<Object> getApproveTree() {
        ReginParams reginParams = getSelectedOrgInfo();
        return ResponseHelper.buildResponse(commonService.getApproveTree(reginParams));
    }

    /**
     * 通过单位类型获取单位列表
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getUnitList")
    @ApiOperation(httpMethod = "GET", value = "通过单位类型获取单位列表", notes = "通过单位类型获取单位列表")
    public ResponseModel<List<Map<String, Object>>> getUnitList(@RequestParam(value = "type") String type) {
        return ResponseHelper.buildResponse(commonService.getUnitListByType(type));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @ApiOperation(httpMethod = "GET", value = "列表全部数据查询", notes = "列表全部数据查询")
    @GetMapping(value = "/getMessage")
    public ResponseModel<Map<String, Object>> getMessage() {
        ReginParams selectedOrgInfo = getSelectedOrgInfo();
        HashMap<String, Object> map = new HashMap<>();
        String companyType = selectedOrgInfo.getCompany().getCompanyType();
        if ("个人主体".equals(companyType)) {
            String[] code = selectedOrgInfo.getCompany().getCompanyCode().split("_");
            map.put("useUnitCreditCode", code.length > 1 ? code[1] : code[0]);
            String[] companyName = selectedOrgInfo.getCompany().getCompanyName().split("_");
            map.put("useUnitName", companyName.length > 1 ? companyName[1] : companyName[0]);
        } else {
            map.put("useUnitCreditCode", selectedOrgInfo.getCompany().getCompanyCode());
            map.put("useUnitName", selectedOrgInfo.getCompany().getCompanyName());
        }
        return ResponseHelper.buildResponse(map);
    }

    /**
     * 查找当前公司下所属的安全管理员
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getSecurityAdmin")
    @ApiOperation(httpMethod = "GET", value = "查找当前公司下所属的安全管理员", notes = "查找当前公司下所属的安全管理员")
    public ResponseModel<List<Map<String, Object>>> getSecurityAdmin(@RequestParam(value = "type") String type,
                                                                     @RequestParam(value = "useUnitCode", required = false) String useUnitCode) {
        return ResponseHelper.buildResponse(commonService.getSecurityAdmin(type, useUnitCode));
    }


    /**
     * 查询安全管理员的基本信息
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getUserInfo")
    @ApiOperation(httpMethod = "GET", value = "查询用户对应联系方式，证件编号", notes = "查询用户对应联系方式，证件编号")
    public ResponseModel<Map<String, Object>> getUserInfo(@RequestParam(value = "sequenceNbr") String sequenceNbr) {
        return ResponseHelper.buildResponse(commonService.getUserInfo(sequenceNbr));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getUserInfoSplit")
    @ApiOperation(httpMethod = "GET", value = "查询安全管理员的基本信息", notes = "查询安全管理员的基本信息")
    public ResponseModel<Map<String, Object>> getUserInfoSplit(@RequestParam(value = "sequenceNbr") String sequenceNbr) {
        if (sequenceNbr.contains("_")) {
            sequenceNbr = sequenceNbr.split("_")[0];
        }
        return ResponseHelper.buildResponse(commonService.getUserInfo(sequenceNbr));
    }

    /**
     * 查询企业的基本信息
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getEnterpriseInfo")
    @ApiOperation(httpMethod = "GET", value = "查询企业的基本信息", notes = "查询企业的基本信息")
    public ResponseModel<Map<String, Object>> getEnterpriseInfo(@RequestParam(value = "useCode") String useCode) {
        return ResponseHelper.buildResponse(commonService.getEnterpriseInfo(useCode));
    }

    /**
     * 查询企业的下面的人员
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getEnterpriseEmployee")
    @ApiOperation(httpMethod = "GET", value = "查询企业的下面的人员", notes = "查询企业的下面的人员")
    public ResponseModel<List<Map<String, Object>>> getEnterpriseEmployee(@RequestParam String unitCode) {
        String companyCode = unitCode.split("_")[0];
        return ResponseHelper.buildResponse(commonService.getEnterpriseEmployee(companyCode));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @ApiOperation(httpMethod = "GET", value = "流程状态枚举列表", notes = "流程状态枚举列表")
    @GetMapping(value = "/flow-status/list")
    public ResponseModel<List<Map<String, Object>>> selectForFlowStatusList() {
        return ResponseHelper.buildResponse(Arrays.stream(FlowStatusEnum.values()).map(this::mapPointTypeToMap).filter(Objects::nonNull).collect(Collectors.toList()));
    }

    private Map<String, Object> mapPointTypeToMap(FlowStatusEnum e) {
        if (FlowStatusEnum.TO_SUBMITTED.getCode() == e.getCode() || FlowStatusEnum.TO_BE_DISCARD.getCode() == e.getCode()) {
            return null;
        }
        Map<String, Object> record = new HashMap<>();
        record.put("key", e.getCode());
        record.put("label", e.getName());
        return record;
    }

    /**
     * 查询监管业务名称列表
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @ApiOperation(httpMethod = "GET", value = "查询监管业务名称列表", notes = "查询监管业务名称列表")
    @GetMapping(value = "/superviseBusinessCategory")
    public ResponseModel<Object> equipTree(String type) {
        return ResponseHelper.buildResponse(commonService.superviseBusinessCategory(type));
    }


    @Autowired
    private IJgInstallationNoticeService iJgInstallationNoticeService;


    /**
     * 业务发起公共接口
     * /jg/jg-installation-notice/save
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/saveOrSubmit")
    @ApiOperation(httpMethod = "POST", value = "业务发起公共接口", notes = "业务发起公共接口")
    public ResponseModel<Object> save(@RequestParam(value = "submitType", required = false) String submitType,
                                      @RequestBody Map<String, Object> map) {
        ReginParams reginParams = getSelectedOrgInfo();
        Object result = commonService.invokeBusinessProcess(submitType, map, reginParams);
        return ResponseHelper.buildResponse(result);
    }

    /**
     * 根据公司id获取公司下人员
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getCompanyUser")
    @ApiOperation(httpMethod = "get", value = "根据公司id获取公司下人员", notes = "根据公司id获取公司下人员")
    public ResponseModel<Object> getCompanyUser(@RequestParam(value = "companyId", required = false) Long companyId) {
        if (ObjectUtils.isEmpty(companyId)) {
            return null;
        }
        Object result = commonService.getCompanyUser(companyId);
        return ResponseHelper.buildResponse(result);
    }

    /**
     * 获取当前登录人企业类型
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getCompanyType")
    @ApiOperation(httpMethod = "GET", value = "获取当前登录人企业类型", notes = "获取当前登录人企业类型")
    public ResponseModel<Object> getCompanyType() {
        return ResponseHelper.buildResponse(iJgInstallationNoticeService.getCompanyType());
    }

    /**
     * 获取管辖分局树
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getCreatTree")
    @ApiOperation(httpMethod = "GET", value = "获取管辖分局树", notes = "获取管辖分局树")
    public ResponseModel<Object> getCreatTree() {
        return ResponseHelper.buildResponse(commonService.getCreatTree());
    }


    /**
     * 获取管辖分局树
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getCreatTreeByInstallation")
    @ApiOperation(httpMethod = "GET", value = "获取管辖分局树安装告知使用", notes = "获取管辖分局树安装告知使用")
    public ResponseModel<Object> getCreatTreeByInstallation(String county) {
        List<LinkedHashMap> creatTree = commonService.getCreatTreeByInstallation();
        String code = county.split("_")[0];
        List<LinkedHashMap> infoByCounty = getInfoByCounty(code, creatTree);
        return ResponseHelper.buildResponse(infoByCounty);
    }

    private List<LinkedHashMap> getInfoByCounty(String county, List<LinkedHashMap> children) {
        for (LinkedHashMap item : children) {
            if (item.get("companyCode").toString().equals(county)) {
                return (List<LinkedHashMap>) item.get("children");
            } else {
                if (!ObjectUtils.isEmpty(item.get("children"))) {
                    List<LinkedHashMap> children1 = getInfoByCounty(county, (List<LinkedHashMap>) item.get("children"));
                    if (!CollectionUtils.isEmpty(children1)) {
                        return children1;
                    }
                }
            }
        }
        return new ArrayList<>();
    }


    /**
     * 公共转办接口
     * /jg/common/transfer
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/transfer")
    @ApiOperation(httpMethod = "POST", value = "公共转办接口", notes = "公共转办接口")
    public ResponseModel<Object> transfer(@RequestBody Map<String, Object> map) {
        Object result = commonService.transfer(map);
        return ResponseHelper.buildResponse(result);
    }


    /**
     * 校验EQUCODE的唯一性
     *
     * @param equCode EQUCODE
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @GetMapping(value = "/checkUniquenessOfEQUCODE")
    @ApiOperation(httpMethod = "GET", value = "校验EQUCODE的唯一性", notes = "校验EQUCODE的唯一性")
    public ResponseModel<Object> checkUniquenessOfEQUCODE(String equCode) {
        Boolean bool = commonService.checkUniquenessOfEQUCODE(equCode);
        return ResponseHelper.buildResponse(bool);
    }


    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @GetMapping(value = "/test-file")
    @ApiOperation(httpMethod = "GET", value = "文件测试", notes = "文件测试")
    public ResponseModel<String> testFile() {
        return ResponseHelper.buildResponse(pdfService.signToPdf("upload/tzs/pdf/1708911268708_temp.pdf", "已作废"));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @GetMapping(value = "/water-print")
    @ApiOperation(httpMethod = "GET", value = "文件测试", notes = "文件测试")
    public ResponseModel<String> signToPdfWaterPrint() {
        return ResponseHelper.buildResponse(pdfService.signToPdfWaterPrint("upload/tzs/pdf/1708911268708_temp.pdf", "已作废"));
    }


    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/equList")
    @ApiOperation(httpMethod = "GET", value = "按照业务类型查询设备种类，工作台使用", notes = "按照业务类型查询设备种类，工作台使用")
    public ResponseModel<List<DictionarieValueModel>> equList(@RequestParam(value = "type") String bizType) {
        // 在平台字典配置的自定义业务的设备种类：格式{bizType}_SBZL,如BF_YZ_SBZL==》移装注销-设备种类
        String suffix = "_SBZL";
        String dictCode = bizType + suffix;
        return ResponseHelper.buildResponse(Systemctl.dictionarieClient.dictValues(dictCode).getResult());
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/equ-category/{categoryCode}/list")
    @ApiOperation(httpMethod = "GET", value = "按照业务类型、设备种类查询设备类别", notes = "按照业务类型、设备种类查询设备类别")
    public ResponseModel<List<DictionarieValueModel>> equCategoryList(@RequestParam(value = "type") String bizType,
                                                                      @PathVariable String categoryCode) {
        // 在平台字典配置的自定义业务的设备种类：格式{bizType}_{categoryCode},如BF_YZ_1000==》移装注销-锅炉
        String dictCode = String.format("%s_%s", bizType, categoryCode);
        return ResponseHelper.buildResponse(Systemctl.dictionarieClient.dictValues(dictCode).getResult());
    }


    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/workbench/permission")
    @ApiOperation(httpMethod = "GET", value = "菜单及登录人信息", notes = "菜单及登录人信息")
    public ResponseModel<Map<String, Object>> permissionTree(@RequestParam String menuAgent,
                                                             @RequestParam String permissionType,
                                                             @RequestParam(required = false) String path) {
        Map<String, Object> result = commonService.getPermissionTreeWithCompany(menuAgent, permissionType, path, getSelectedOrgInfo().getCompany());
        return ResponseHelper.buildResponse(result);
    }

    /**
     * 查询当前登录用户公司下的管理员对应联系方式
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/getUserPhone")
    @ApiOperation(httpMethod = "GET", value = "查询当前登录用户公司下的管理员对应联系方式", notes = "查询当前登录用户公司下的管理员对应联系方式，证件编号")
    public ResponseModel<Map<String, Object>> getUserPhone(@RequestParam(value = "name") String name) {
        return ResponseHelper.buildResponse(commonService.getUserPhone(name));
    }



    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @GetMapping(value = "/use-flag-test")
    @ApiOperation(httpMethod = "GET", value = "使用标志生成测试", notes = "使用标志生成测试")
    public ResponseModel<String> getUserPhone(HttpServletResponse response) {
        UseFlagParamDto useFlagParamDto = buildUseFlagParam();
        commonService.useFlagGenerate(useFlagParamDto, response);
        return ResponseHelper.buildResponse("succese");
    }

    private UseFlagParamDto buildUseFlagParam(){
        UseFlagParamDto useFlagParamDto = new UseFlagParamDto();
        useFlagParamDto.setEquList("设备种类名称");
        useFlagParamDto.setEquListCode("0000");
        useFlagParamDto.setEquipCode("10001101");
        useFlagParamDto.setEquipDefine("设备品种");
        useFlagParamDto.setUseUnitName("使用单位名称");
        useFlagParamDto.setEquipCategory("设备类别名称");
        useFlagParamDto.setUseInnerCode("单位内编号");
        useFlagParamDto.setReceiveOrgName("登记机关");
        useFlagParamDto.setInspectionUnitName("检验机构名称");
        useFlagParamDto.setUseRegistrationCode("11111");
        useFlagParamDto.setNextInspectionDate(new Date());
        useFlagParamDto.setMaintenanceUnitName("陕西国翔锅炉集团有限公司天水安装销售公司");
        useFlagParamDto.setEmergencyTel("17792961611");
        useFlagParamDto.setCarNumber("车牌号");
        useFlagParamDto.setFactoryNumber("产品编号");
        useFlagParamDto.setCylinderNumOrVolume("12");
        useFlagParamDto.setFillingMedium("氮气");
        useFlagParamDto.setApplyNo("101");
        return useFlagParamDto;
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/vehicleInformationDict")
    @ApiOperation(httpMethod = "GET", value = "按照类型查询车载气瓶相关的字典值", notes = "按照类型查询车载气瓶相关的字典值")
    public ResponseModel<List<DictionarieValueModel>> vehicleInformationDict(@RequestParam(value = "type") String ype) {
        return ResponseHelper.buildResponse(Systemctl.dictionarieClient.dictValues(ype).getResult());
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @GetMapping(value = "/refreshTheDetailsDataOfCompletedUsageRegistration")
    @ApiOperation(httpMethod = "GET", value = "刷新历史数据脚本：since之前完成使用登记的单子，向历史表中刷入设备的维保信息和安装信息",
            notes = "刷新历史数据脚本：since之前完成使用登记的单子，向历史表中刷入设备的维保信息和安装信息")
    public ResponseModel<Object> refreshTheDetailsDataOfCompletedUsageRegistration(@RequestParam(value = "since") String since) {
        // 查询时间在since之前的已经完成的使用登记单子的seqIds
        List<String> seqLists = commonService.refreshTheDetailsDataOfCompletedUsageRegistration(since);
        // 遍历查询历史表，补充changeData中维保信息和安装信息的数据
        seqLists.forEach(sequenceNbr ->{
            LambdaQueryWrapper<JgRegistrationHistory> lambda = new QueryWrapper<JgRegistrationHistory>().lambda();
            lambda.eq(JgRegistrationHistory::getCurrentDocumentId, sequenceNbr);
            JgRegistrationHistory jgRegistrationHistory = jgRegistrationHistoryService.getBaseMapper().selectOne(lambda);
            JSONObject jsonObject = JSONObject.parseObject(jgRegistrationHistory.getChangeData());
            String equipId = String.valueOf(jsonObject.get("equipId"));
            if (!ObjectUtils.isEmpty(equipId)){
                // 维保信息
                Map<String, Object> maintenanceDetail = jgUseRegistrationMapper.getMaintenanceDetail(equipId);
                if (!ObjectUtils.isEmpty(maintenanceDetail)){
                    jsonObject.putAll(maintenanceDetail);
                }
                // 安装信息
                Map<String, Object> installDetail = jgUseRegistrationMapper.getiInstallDetail(equipId);
                if (!ObjectUtils.isEmpty(installDetail)){
                    jsonObject.putAll(installDetail);
                }
                jgRegistrationHistory.setChangeData(JSON.toJSONString(jsonObject));
            }
            jgRegistrationHistoryService.updateById(jgRegistrationHistory);
        });
        return ResponseHelper.buildResponse("success");
    }



    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PutMapping(value = "/test-code")
    @ApiOperation(httpMethod = "PUT", value = "code生成测试", notes = "code生成测试")
    public ResponseModel<String> codeTest(@ApiParam("设备种类") @RequestParam String equList,
                                          @ApiParam("设备类别") @RequestParam String equCategory,
                                          @ApiParam("设备品种") @RequestParam(required = false) String equDefine,
                                          @ApiParam("接收机构行政区划") @RequestParam String receiveCompanyCode,
                                          @ApiParam("制造日期") @RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd") Date produceDate) {
        CodeGenerateDto codeGenerateDto = new CodeGenerateDto();
        codeGenerateDto.setEquList(equList);
        codeGenerateDto.setEquCategory(equCategory);
        codeGenerateDto.setEquDefine(equDefine);
        codeGenerateDto.setProduceDate(produceDate);
        codeGenerateDto.setReceiveCompanyCode(receiveCompanyCode);
        return ResponseHelper.buildResponse(codeUtil.generateEquipmentCode(codeGenerateDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PutMapping(value = "/init-code")
    @ApiOperation(httpMethod = "PUT", value = "设备代码初始化-运维专用", notes = "设备代码初始化-运维专用")
    public ResponseModel<String> initEquCode(@ApiParam("设备种类") @RequestParam String equList,
                                             @ApiParam("设备类别") @RequestParam String equCategory,
                                             @ApiParam("设备品种") @RequestParam(required = false) String equDefine,
                                             @ApiParam("接收机构行政区划") @RequestParam String receiveCompanyCode,
                                             @ApiParam("制造日期") @RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd") Date produceDate,
                                             @ApiParam("重置的序列号") @RequestParam(required = false, defaultValue = "0") Long intNumber){
        CodeGenerateDto codeGenerateDto = new CodeGenerateDto();
        codeGenerateDto.setEquList(equList);
        codeGenerateDto.setEquCategory(equCategory);
        codeGenerateDto.setEquDefine(equDefine);
        codeGenerateDto.setProduceDate(produceDate);
        codeGenerateDto.setReceiveCompanyCode(receiveCompanyCode);
        codeUtil.initCode(codeGenerateDto, intNumber);
        return ResponseHelper.buildResponse("succese");
    }


}
